@@ -5,8 +5,6 @@ import click
 import time
 import os
 import re
-from time import strftime, localtime
-from multiprocessing import Lock
 import traceback
 import json
 
@@ -16,20 +14,15 @@ from .commandanalyze import execute
 from .commandanalyze import CommandNotFound
 from .sqlcliexception import SQLCliException
 from SQLCliODBC import SQLCliODBCException
+from .sqlclijdbcapi import SQLCliJDBCException
 
 
 class SQLExecute(object):
     def __init__(self):
         self.conn = None                    # 数据库连接
-        self.logfile = None                 # 打印的日志文件
         self.sqlscript = None               # 需要执行的SQL脚本
         self.SQLMappingHandler = None       # SQL重写处理
         self.SQLOptions = None              # 程序处理参数
-        self.Console = False                # 屏幕输出Console
-        self.logger = None                  # 日志输出
-        self.m_PerfFileLocker = None        # 进程锁, 用来在输出perf文件的时候控制并发写文件
-        self.SQLPerfFile = None             # SQLPerf文件
-        self.SQLPerfFileHandle = None       # SQLPerf文件句柄
         self.SQLCliHandler = None
 
         # 记录最后SQL返回的结果
@@ -39,9 +32,6 @@ class SQLExecute(object):
         # 当前Executeor的WorkerName
         self.WorkerName = None
 
-        # 程序Spool输出句柄
-        self.spoolfile = None
-
         # 程序Echo输出句柄
         self.echofile = None
 
@@ -109,13 +99,35 @@ class SQLExecute(object):
 
     def run(self, statement, p_sqlscript=None):
         """
-        返回结果包含5个方面的内容
-        (title, rows, headers, columntypes, status).
-        title       表头信息
-        rows        结果数据集
-        headers     结果集的header定义，列名信息
-        columntypes 列类型，字符串格式
-        status       返回结果汇总消息
+        返回的结果可能是多种类型，处理的时候需要根据type的结果来判断具体的数据格式：
+        SQL解析结果：
+        {
+            "type": "parse",
+            "rawsql": "原始SQL语句",
+            "formattedsql": "被排版后的SQL语句（包含注释信息）",
+            "rewrotedsql": "被改写后的SQL语句（已经被排版过），如果不存在改写，则不存在",
+            "script"："SQL当前执行的脚本名称"
+        }
+        SQL执行结果：
+        {
+            "type": "result",
+            "title": "表头信息",
+            "rows": "结果数据集",
+            "headers": "结果集的header定义，列名信息",
+            "columntypes": "列类型，字符串格式",
+            "status": "返回结果汇总消息"
+        }
+        SQL错误信息：
+        {
+            "type": "error",
+            "message": "错误描述"
+        }
+        SQL回显信息：
+        {
+            "type": "error",
+            "message": "错误描述",
+            "script"："SQL当前执行的脚本名称"
+        }
 
         支持的SQLHint包括:
         -- [Hint]  order           -- SQLCli将会把随后的SQL语句进行排序输出，原程序的输出顺序被忽略
@@ -128,7 +140,7 @@ class SQLExecute(object):
         # Remove spaces and EOL
         statement = statement.strip()
         if not statement:  # Empty string
-            yield None, None, None, None, None
+            return
 
         # 分析SQL语句
         (ret_bSQLCompleted, ret_SQLSplitResults,
@@ -136,23 +148,20 @@ class SQLExecute(object):
         for m_nPos in range(0, len(ret_SQLSplitResults)):
             m_raw_sql = ret_SQLSplitResults[m_nPos]                  # 记录原始SQL
 
-            # 如果当前是在回显一个文件，并且不是echo off，则不再做任何处理
+            # 如果当前是在回显一个文件，则不再做任何处理，直接返回
             if self.echofile is not None and \
                     not re.match(r'echo\s+off', m_raw_sql, re.IGNORECASE):
-                if self.SQLOptions.get("ECHO").upper() == 'ON' and self.logfile is not None:
-                    click.echo(m_raw_sql, file=self.logfile)
-                if self.SQLOptions.get("ECHO").upper() == 'ON' and self.spoolfile is not None:
-                    for m_SpoolFileHandler in self.spoolfile:
-                        click.echo(m_raw_sql, file=m_SpoolFileHandler)
-                if self.logger is not None:
-                    if m_raw_sql is not None:
-                        self.logger.info(m_raw_sql)
-                click.echo(m_raw_sql, file=self.echofile)
-                yield None, None, None, None, m_raw_sql
+                yield {
+                    "type": "echo",
+                    "message": m_raw_sql,
+                    "script": p_sqlscript
+                }
                 continue
 
             sql = m_raw_sql                                          # 当前要被执行的SQL，这个SQL可能被随后的注释或者替换规则改写
             m_CommentSQL = ret_SQLSplitResultsWithComments[m_nPos]   # 记录带有注释信息的SQL
+            m_FormattedSQL = None                                    # 已经被格式化了的SQL语句
+            m_RewrotedSQL = []                                       # SQL可能会被多次改写
 
             # 分析SQLHint信息
             m_SQLHint = ret_SQLHints[m_nPos]                         # SQL提示信息，其中Scenario用作日志处理
@@ -162,57 +171,89 @@ class SQLExecute(object):
                 else:
                     self.SQLScenario = m_SQLHint['SCENARIO']
 
-            # 如果打开了回显，并且指定了输出文件，则在输出文件里显示SQL语句
-            if self.SQLOptions.get("ECHO").upper() == 'ON' \
-                    and len(m_CommentSQL.strip()) != 0 and self.logfile is not None:
-                if self.SQLOptions.get("SILENT").upper() == 'OFF':
-                    click.echo(SQLFormatWithPrefix(m_CommentSQL), file=self.logfile)
-            if self.SQLOptions.get("ECHO").upper() == 'ON' \
-                    and len(m_CommentSQL.strip()) != 0 and self.spoolfile is not None:
-                # 在spool文件中，不显示spool off的信息，以避免log比对中的不必要内容
-                if not re.match(r'spool\s+.*', m_CommentSQL.strip(), re.IGNORECASE):
-                    if self.SQLOptions.get("SILENT").upper() == 'OFF':
-                        for m_SpoolFileHandler in self.spoolfile:
-                            click.echo(SQLFormatWithPrefix(m_CommentSQL), file=m_SpoolFileHandler)
-
-            # 在logger中显示执行的SQL
-            if self.logger is not None:
-                m_LoggerMessage = SQLFormatWithPrefix(m_CommentSQL)
-                if m_LoggerMessage is not None:
-                    if self.SQLOptions.get("SILENT").upper() == 'OFF':
-                        self.logger.info(m_LoggerMessage)
-
-            # 如果运行在脚本模式下，需要在控制台额外回显SQL
-            # 非脚本模式下，由于是用户自行输入，所以不需要回显输入的SQL
-            if p_sqlscript is not None:
-                if self.SQLOptions.get("SILENT").upper() == 'OFF':
-                    click.echo(SQLFormatWithPrefix(m_CommentSQL), file=self.Console)
+            # 记录包含有注释信息的SQL
+            m_FormattedSQL = SQLFormatWithPrefix(m_CommentSQL)
 
             # 如果打开了回显，并且指定了输出文件，且SQL被改写过，输出改写后的SQL
             if self.SQLOptions.get("SQLREWRITE").upper() == 'ON':
                 old_sql = sql
                 sql = self.SQLMappingHandler.RewriteSQL(p_sqlscript, old_sql)
                 if old_sql != sql:    # SQL已经发生了改变
-                    if self.SQLOptions.get("SILENT").upper() == 'ON':
-                        # SILENT模式下不打印任何内容出来
-                        pass
-                    else:
-                        if self.SQLOptions.get("ECHO").upper() == 'ON' and self.logfile is not None:
-                            click.echo(SQLFormatWithPrefix(
-                                "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.logfile)
-                        if self.SQLOptions.get("ECHO").upper() == 'ON' and self.spoolfile is not None:
-                            for m_SpoolFileHandler in self.spoolfile:
-                                click.echo(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '),
-                                           file=m_SpoolFileHandler)
-                        if self.logger is not None:
-                            self.logger.info(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '))
-                        click.echo(SQLFormatWithPrefix(
-                            "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.Console)
+                    # 记录被SQLMapping改写的SQL
+                    m_RewrotedSQL.append(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '))
 
-            # 如果是空语句，不在执行
+            # 如果是空语句，不需要执行，但可能是完全注释行
             if len(sql.strip()) == 0:
+                # 返回SQL的解析信息
+                yield {
+                    "type": "parse",
+                    "rawsql": m_raw_sql,
+                    "formattedsql": m_FormattedSQL,
+                    "rewrotedsql": m_RewrotedSQL,
+                    "script": p_sqlscript
+                }
                 continue
 
+            matchObj = re.match(r"^show\s+table\s+storage\s+(.*)$", sql, re.IGNORECASE)
+            if matchObj:
+                m_TableName = matchObj.group(1)
+                m_Location = ""
+                for m_Result in self.run("show table properties " + m_TableName):
+                    if m_Result["type"] == "result":
+                        if len(m_Result["rows"]) != 0:
+                            m_Location = m_Result["rows"][len(m_Result["rows"]) - 1][1]
+                            break
+                if m_Location != "":
+                    m_HostCommand = "hdfs fsck " + m_Location + " -files -blocks -locations"
+                    m_Result = []
+                    m_Status = ""
+                    for m_OSResult in self.run("host " + m_HostCommand):
+                        m_HostBlockInfo = {}
+                        if m_OSResult["type"] == "result":
+                            m_StoargeInfo = m_OSResult["status"].splitlines()
+                            m_BlockCount = 0
+                            for m_line in m_StoargeInfo:
+                                matchObj = re.search(r"bytes, (\d+) block\(s\):\s+OK", m_line)
+                                if matchObj:
+                                    m_BlockCount = int(matchObj.group(1))
+                                matchObj = re.search(r"DatanodeInfoWithStorage\[(.*?):", m_line)
+                                if matchObj:
+                                    m_Host = str(matchObj.group(1)).strip()
+                                    if m_Host in m_HostBlockInfo.keys():
+                                        m_HostBlockInfo[m_Host] = m_HostBlockInfo[m_Host] + m_BlockCount
+                                    else:
+                                        m_HostBlockInfo[m_Host] = m_BlockCount
+                                matchObj = re.search(r"Status:", m_line)
+                                if matchObj:
+                                    m_Status = m_Status + m_line + "\n"
+                                matchObj = re.search(r"Total size:", m_line)
+                                if matchObj:
+                                    m_Status = m_Status + m_line + "\n"
+                                matchObj = re.search(r"Total dirs:", m_line)
+                                if matchObj:
+                                    m_Status = m_Status + m_line + "\n"
+                                matchObj = re.search(r"Total files:", m_line)
+                                if matchObj:
+                                    m_Status = m_Status + m_line + "\n"
+                        for HostIP, HostBlockCount in m_HostBlockInfo.items():
+                            m_Result.append((HostIP, HostBlockCount))
+                    yield {
+                        "type": "parse",
+                        "rawsql": m_raw_sql,
+                        "formattedsql": m_FormattedSQL,
+                        "rewrotedsql": m_RewrotedSQL,
+                        "script": p_sqlscript
+                    }
+                    yield {
+                        "type": "result",
+                        "title": "HDFS Storage Information for " + m_TableName,
+                        "rows": m_Result,
+                        "headers": ["Node", "Block(s)"],
+                        "columntypes": ["str", "int"],
+                        "status": m_Status
+                    }
+                return
+
             # 检查SQL中是否包含特殊内容，如果有，改写SQL
             # 特殊内容都有：
             # 1. ${LastSQLResult(.*)}       # .* JQ Parse Pattern
@@ -229,17 +270,8 @@ class SQLExecute(object):
                     # SILENT模式下不打印任何日志
                     pass
                 else:
-                    if self.SQLOptions.get("ECHO").upper() == 'ON' and self.logfile is not None:
-                        click.echo(SQLFormatWithPrefix(
-                            "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.logfile)
-                    if self.SQLOptions.get("ECHO").upper() == 'ON' and self.spoolfile is not None:
-                        for m_SpoolFileHandler in self.spoolfile:
-                            click.echo(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '),
-                                       file=m_SpoolFileHandler)
-                    if self.logger is not None:
-                        self.logger.info(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '))
-                    click.echo(SQLFormatWithPrefix(
-                        "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.Console)
+                    # 记录被JQ表达式改写的SQL
+                    m_RewrotedSQL.append(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '))
 
             # ${var}
             bMatched = False
@@ -268,18 +300,17 @@ class SQLExecute(object):
                 else:
                     break
             if bMatched:
-                if self.SQLOptions.get("ECHO").upper() == 'ON' and self.logfile is not None:
-                    # SQL已经发生了改变, 会将改变后的SQL信息在屏幕上单独显示出来
-                    click.echo(SQLFormatWithPrefix(
-                        "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.logfile)
-                if self.SQLOptions.get("ECHO").upper() == 'ON' and self.spoolfile is not None:
-                    for m_SpoolFileHandler in self.spoolfile:
-                        click.echo(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '),
-                                   file=m_SpoolFileHandler)
-                if self.logger is not None:
-                    self.logger.info(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '))
-                click.echo(SQLFormatWithPrefix(
-                    "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.Console)
+                # 记录被变量信息改写的SQL
+                m_RewrotedSQL.append(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '))
+
+            # 返回SQL的解析信息
+            yield {
+                "type": "parse",
+                "rawsql": m_raw_sql,
+                "formattedsql": m_FormattedSQL,
+                "rewrotedsql": m_RewrotedSQL,
+                "script": p_sqlscript
+            }
 
             # 记录命令开始时间
             start = time.time()
@@ -287,8 +318,10 @@ class SQLExecute(object):
             # 执行SQL
             try:
                 # 首先尝试这是一个特殊命令，如果返回CommandNotFound，则认为其是一个标准SQL
-                for (title, result, headers, columntypes, status) in execute(self.SQLCliHandler,  sql):
-                    yield title, result, headers, columntypes, status
+                for m_Result in execute(self.SQLCliHandler,  sql):
+                    if "type" not in m_Result.keys():
+                        m_Result.update({"type": "result"})
+                    yield m_Result
             except CommandNotFound:
                 # 进入到SQL执行阶段, 开始执行SQL语句
                 if self.conn:
@@ -300,13 +333,17 @@ class SQLExecute(object):
                         raise SQLCliException("Not Connected. ")
                     else:
                         self.LastJsonSQLResult = None
-                        yield None, None, None, None, "Not connected. "
+                        yield {
+                            "type": "error",
+                            "message": "Not connected. "
+                        }
+                        return
 
                 # 执行正常的SQL语句
                 if self.cur is not None:
                     try:
                         if "SQLCLI_DEBUG" in os.environ:
-                            click.secho("DEBUG-SQL=[" + str(sql) + "]", file=self.logfile)
+                            print("DEBUG-SQL=[" + str(sql) + "]")
                         # 执行SQL脚本
                         if "SQL_DIRECT" in m_SQLHint.keys():
                             self.cur.execute_direct(sql)
@@ -423,10 +460,23 @@ class SQLExecute(object):
 
                             # 返回SQL结果
                             if self.SQLOptions.get('TERMOUT').upper() != 'OFF':
-                                yield title, result, headers, columntypes, status
+                                yield {
+                                    "type": "result",
+                                    "title": title,
+                                    "rows": result,
+                                    "headers": headers,
+                                    "columntypes": columntypes,
+                                    "status": status
+                                }
                             else:
-                                yield title, [], headers, columntypes, status
-
+                                yield {
+                                    "type": "result",
+                                    "title": title,
+                                    "rows": [],
+                                    "headers": headers,
+                                    "columntypes": columntypes,
+                                    "status": status
+                                }
                             if not m_FetchStatus:
                                 break
                     except SQLCliODBCException as oe:
@@ -435,16 +485,14 @@ class SQLExecute(object):
                         for m_ErrorPrefix in ('ERROR:',):
                             if m_SQL_ErrorMessage.startswith(m_ErrorPrefix):
                                 m_SQL_ErrorMessage = m_SQL_ErrorMessage[len(m_ErrorPrefix):].strip()
-
-                        # 发生了SQL语法错误
+                        # 发生了ODBC的SQL语法错误
                         if self.SQLOptions.get("WHENEVER_SQLERROR") == "EXIT":
                             raise SQLCliException(m_SQL_ErrorMessage)
                         else:
-                            self.LastJsonSQLResult = None
-                            yield None, None, None, None, m_SQL_ErrorMessage
-                    except Exception as e:
+                            yield {"type": "error", "message": m_SQL_ErrorMessage}
+                    except SQLCliJDBCException as je:
                         m_SQL_Status = 1
-                        m_SQL_ErrorMessage = str(e).strip()
+                        m_SQL_ErrorMessage = str(je).strip()
                         for m_ErrorPrefix in ('java.sql.SQLSyntaxErrorException:',
                                               "java.sql.SQLException:",
                                               "java.sql.SQLInvalidAuthorizationSpecException:",
@@ -491,13 +539,25 @@ class SQLExecute(object):
                             if m_ErrorMessageHasChanged:
                                 m_SQL_ErrorMessage = "\n".join(m_SQL_MultiLineErrorMessage)
 
-                        # 发生了SQL语法错误
+                        # 发生了JDBC的SQL语法错误
                         if self.SQLOptions.get("WHENEVER_SQLERROR") == "EXIT":
                             raise SQLCliException(m_SQL_ErrorMessage)
                         else:
-                            self.LastJsonSQLResult = None
-                            yield None, None, None, None, m_SQL_ErrorMessage
-            except (SQLCliException, SQLCliODBCException) as e:
+                            yield {"type": "error", "message": m_SQL_ErrorMessage}
+                    except Exception as e:
+                        m_SQL_Status = 1
+                        m_SQL_ErrorMessage = str(e).strip()
+
+                        # 发生了其他SQL语法错误
+                        if self.SQLOptions.get("WHENEVER_SQLERROR") == "EXIT":
+                            raise SQLCliException(m_SQL_ErrorMessage)
+                        else:
+                            yield {"type": "error", "message": m_SQL_ErrorMessage}
+            except EOFError:
+                # EOFError是程序退出的标志，退出应用程序
+                raise EOFError
+            except Exception as e:
+                # 只有在WHENEVER_SQLERROR==EXIT的时候，才会由前面的错误代码抛出错误到这里
                 m_SQL_Status = 1
                 m_SQL_ErrorMessage = str(e).strip()
                 # 如果要求出错退出，就立刻退出，否则打印日志信息
@@ -508,7 +568,7 @@ class SQLExecute(object):
                         print('traceback.print_exc():\n%s' % traceback.print_exc())
                         print('traceback.format_exc():\n%s' % traceback.format_exc())
                     self.LastJsonSQLResult = None
-                    yield None, None, None, None, m_SQL_ErrorMessage
+                    yield {"type": "error", "message": m_SQL_ErrorMessage}
 
             # 如果需要，打印语句执行时间
             end = time.time()
@@ -516,26 +576,18 @@ class SQLExecute(object):
 
             # 记录SQL日志信息
             if self.SQLOptions.get("SILENT").upper() == 'OFF':
-                self.Log_Perf(
-                    {
-                        "StartedTime": start,
-                        "elapsed": self.LastElapsedTime,
-                        "RAWSQL": m_raw_sql,
-                        "SQL": sql,
-                        "SQLStatus": m_SQL_Status,
-                        "ErrorMessage": m_SQL_ErrorMessage,
-                        "thread_name": self.WorkerName,
-                        "Scenario": self.SQLScenario,
-                        "Transaction": self.SQLTransaction
-                    }
-                )
-
-            if self.SQLOptions.get('TIMING').upper() == 'ON':
-                if sql.strip().upper() not in ('EXIT', 'QUIT'):
-                    yield None, None, None, None, 'Running time elapsed: %9.2f Seconds' % self.LastElapsedTime
-            if self.SQLOptions.get('TIME').upper() == 'ON':
-                if sql.strip().upper() not in ('EXIT', 'QUIT'):
-                    yield None, None, None, None, 'Current clock time  :' + strftime("%Y-%m-%d %H:%M:%S", localtime())
+                yield {
+                    "type": "statistics",
+                    "StartedTime": start,
+                    "elapsed": self.LastElapsedTime,
+                    "RAWSQL": m_raw_sql,
+                    "SQL": sql,
+                    "SQLStatus": m_SQL_Status,
+                    "ErrorMessage": m_SQL_ErrorMessage,
+                    "thread_name": self.WorkerName,
+                    "Scenario": self.SQLScenario,
+                    "Transaction": self.SQLTransaction
+                }
 
     def get_result(self, cursor, rowcount):
         """Get the current result's data from the cursor."""
@@ -627,62 +679,3 @@ class SQLExecute(object):
         else:
             status = None
         return title, result, headers, columntypes, status, m_FetchStatus, rowcount
-
-    # 记录PERF信息，
-    def Log_Perf(self, p_SQLResult):
-        # 开始时间         StartedTime
-        # 消耗时间         elapsed
-        # SQL的前20个字母  SQLPrefix
-        # 运行状态         SQLStatus
-        # 错误日志         ErrorMessage
-        # 线程名称         thread_name
-
-        # 如果没有打开性能日志记录文件，直接跳过
-        if self.SQLPerfFile is None:
-            return
-
-        # 初始化文件加锁机制
-        if self.m_PerfFileLocker is None:
-            self.m_PerfFileLocker = Lock()
-
-        # 多进程，多线程写入，考虑锁冲突
-        try:
-            self.m_PerfFileLocker.acquire()
-            if not os.path.exists(self.SQLPerfFile):
-                # 如果文件不存在，创建文件，并写入文件头信息
-                self.SQLPerfFileHandle = open(self.SQLPerfFile, "a", encoding="utf-8")
-                self.SQLPerfFileHandle.write("Script\tStarted\telapsed\tRAWSQL\tSQL\t"
-                                             "SQLStatus\tErrorMessage\tworker_name\t"
-                                             "Scenario\tTransaction\n")
-                self.SQLPerfFileHandle.close()
-
-            # 对于多线程运行，这里的thread_name格式为JOB_NAME#副本数-完成次数
-            # 对于单线程运行，这里的thread_name格式为固定的MAIN
-            m_ThreadName = str(p_SQLResult["thread_name"])
-
-            # 打开Perf文件
-            self.SQLPerfFileHandle = open(self.SQLPerfFile, "a", encoding="utf-8")
-            # 写入内容信息
-            if self.sqlscript is None:
-                m_SQL_Script = "Console"
-            else:
-                m_SQL_Script = str(os.path.basename(self.sqlscript))
-            self.SQLPerfFileHandle.write(
-                m_SQL_Script + "\t" +
-                time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(p_SQLResult["StartedTime"])) + "\t" +
-                "%8.2f" % p_SQLResult["elapsed"] + "\t" +
-                str(p_SQLResult["RAWSQL"]).replace("\n", " ").replace("\t", "    ") + "\t" +
-                str(p_SQLResult["SQL"]).replace("\n", " ").replace("\t", "    ") + "\t" +
-                str(p_SQLResult["SQLStatus"]) + "\t" +
-                str(p_SQLResult["ErrorMessage"]).replace("\n", " ").replace("\t", "    ") + "\t" +
-                str(m_ThreadName) + "\t" +
-                str(p_SQLResult["Scenario"]) + "\t" +
-                str(p_SQLResult["Transaction"]) +
-                "\n"
-            )
-            self.SQLPerfFileHandle.flush()
-            self.SQLPerfFileHandle.close()
-        except Exception as ex:
-            print("Internal error:: perf file write not complete. " + repr(ex))
-        finally:
-            self.m_PerfFileLocker.release()
